import re
import string
from java.util import Vector
from hades.manager import DesignManager


def setEditor(edi):
    ''' <None> setEditor(editor)
    The current editor-object is replaced to the editor-argument.
    '''
    global editor
    editor = edi


def getEditor():
    ''' editor getEditor()
    getEditor return the current editor-object.
    '''
    global editor
    return editor


def getEditorByDesign(design_name):
    ''' editor  getEditorByDesign(design_name)
    getEditorByDesign return the editor, thats contains the design 
    specified by his name.
    '''
    editor = getEditor()
    editor_list = editor.getEditors()    
 
    while(editor_list.hasMoreElements()):
	e = editor_list.nextElement()
	if (e.getDesign().getName() == design_name):
	    jp_design_os.setEditor(e)

	    
def updateDesign():
    ''' <None> updateDesign()
    updateDesign update the interpreters design-object with the design 
    get from the editor-object.
    ''' 
    global design
    editor = getEditor()
    design = editor.getDesign()


def getDesign():
    ''' <design> getDesign()
    getDesign return the current design-object, if there
    is no interpreter design-object, the "updateDesign()" method
    is called.
    '''
    global design
    try: 
	return design
    except NameError:
	updateDesign()
	return design


def setDesign(d):
    ''' <None> setDesign(design)
    setDesign change the global interpreter design-object.
    '''
    global design
    design = d


def openDesign(resourcename):
    ''' <None> openDesign(resourcename)
    openDesign try to open a hades-file (.hds) specified 
    by the resourcename in a new HADES-Editor and update the
    design. 
    For examble:
    - openDesign("/hades/examples/simple/dlatch.hds")
    '''
    editor = getEditor()
    editor.doOpenDesign(resourcename, 0)
    updateDesign()

def infoDesign():
    ''' <None> infoDesign()
    infoDesign displays all the info about the design and its components.
    '''
    d = getDesign()
    print d
    e = d.getComponents()
    while(e.hasMoreElements()):
	tmp = e.nextElement()
	print "%-20s - %-20s" %(tmp.getName(), tmp.getFullName())
	

def pwd():
    ''' <string> pwd()
    pwd displays and return the current design-path.
    '''
    path = getDesign()
    e = path.getComponents()
    if (e.hasMoreElements()):
	tmp = e.nextElement()
	tmp_path = tmp.getFullName()
	tmp_list = string.split(tmp_path, '/')
	path = tmp_list[:-1]
	print path
    return path


def ls(pattern=""):
    ''' <vector> ls([pattern=""])
    ls diplays all objects from the design-path which
    are matched with the pattern
    For example:
    - ls("design")
    '''
    p = re.compile(pattern)
    
    d = getDesign()
    e = d.getComponents()
    while (e.hasMoreElements()):
	tmp = e.nextElement()
	tmp_name = tmp.getName()
        tmp_info = tmp.toString()
	m = p.match(tmp_name)
	n = p.match(tmp_info)
	if (m or n):
	    print "%-20s - %20s" %(str(tmp.getName()), str(tmp))
	
   	
def cd(design_name):
    ''' <None> cd(design_name)
    cd change the design-path, to the "design_name".
    For example:
    - cd("..") change to the parent design
    - cd("/") change to the topdesign
    '''
    d = getDesign()
    if (design_name == ".."):
	design = d.getParent()
    elif (design_name == "/"):
	design = editor.getDesign()
    else:
        design = d.getComponent(design_name)
    setDesign(design)


def getByName(name):
    ''' <None> get(name)
    getByName return the sim-object specified by his name.
    '''
    d = getDesign()
    sim_object = d.getComponent(name)
    return sim_object


def get(pattern=""):
    ''' <vector> ls([pattern=""])
    get return all objects from the design-path which
    are matched with the pattern
    For example:
    - all_designs = get("design")
    '''
    result = Vector()
    p = re.compile(pattern)
    
    d = getDesign()
    e = d.getComponents()
    while (e.hasMoreElements()):
	tmp = e.nextElement()
	tmp_name = tmp.getName()
        tmp_info = tmp.toString()
	m = p.match(tmp_name)
	n = p.match(tmp_info)
	if (m or n):
	    result.addElement(tmp)
    if(len(result)<=1):
        return result[0]
    else:
        return result
